"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryComplexityError = void 0;
const assert_1 = __importDefault(require("assert"));
const graphql_1 = require("graphql");
const graphql_query_complexity_1 = require("graphql-query-complexity");
class QueryComplexityError extends graphql_1.GraphQLError {
    constructor(complexity, maximumComplexity) {
        super(`Query is too complex. Requested complexity ${complexity} is greater than maximum allowed ${maximumComplexity}.`, {
            extensions: {
                code: 'QUERY_TOO_COMPLEX',
                complexity,
                maximumComplexity,
            },
        });
    }
}
exports.QueryComplexityError = QueryComplexityError;
function plugin(options) {
    (0, assert_1.default)(typeof options === 'object' && options !== null, 'options is required');
    (0, assert_1.default)(Array.isArray(options.estimators), 'options.estimators must be an array.');
    (0, assert_1.default)(typeof options.maximumComplexity === 'number' &&
        options.maximumComplexity > 0, 'options.maximumComplexity must be a positive number.');
    return {
        requestDidStart({ schema }) {
            return __awaiter(this, void 0, void 0, function* () {
                return {
                    didResolveOperation({ request, document }) {
                        return __awaiter(this, void 0, void 0, function* () {
                            const complexity = (0, graphql_query_complexity_1.getComplexity)({
                                schema,
                                operationName: request.operationName,
                                query: document,
                                estimators: options.estimators,
                                variables: request.variables,
                            });
                            if (complexity > options.maximumComplexity) {
                                throw new QueryComplexityError(complexity, options.maximumComplexity);
                            }
                        });
                    },
                };
            });
        },
    };
}
exports.default = plugin;
//# sourceMappingURL=ApolloServerPluginQueryComplexity.js.map